export declare type Search = Record<string, string | string[]>;
export declare type Params = Record<string, string | string[] | undefined>;
export declare type Subscription = (location: Location) => void;
export declare type Segment = {
    name: string;
    param: boolean;
};
export declare type Matcher = {
    name: string;
    segments: Segment[];
    search: Record<string, "unique" | "multiple">;
    hash?: string;
    finite: boolean;
    ranking: number;
};
export declare type Location = Readonly<{
    url: string;
    path: readonly string[];
    search: Readonly<Search>;
    hash?: string;
}>;
declare type Split<Value extends string, Separator extends string> = Value extends `${infer Head}${Separator}${infer Tail}` ? [Head, ...Split<Tail, Separator>] : Value extends Separator ? [] : [Value];
declare type ExtractPathParams<Path extends string, Parts = Split<Path, "/">> = Parts extends [infer Head, ...infer Tail] ? Head extends `:${infer Name}` ? {
    [K in Name]: string;
} & ExtractPathParams<Path, Tail> : ExtractPathParams<Path, Tail> : {};
declare type ExtractSearchParams<Search extends string, Parts = Split<Search, "&">> = Parts extends [infer Head, ...infer Tail] ? Head extends `:${infer Name}[]` ? {
    [K in Name]?: string[];
} & ExtractSearchParams<Search, Tail> : Head extends `:${infer Name}` ? {
    [K in Name]?: string;
} & ExtractSearchParams<Search, Tail> : ExtractSearchParams<Search, Tail> : {};
declare type ExtractHashParam<Value extends string> = Value extends `:${infer Name}` ? {
    [K in Name]?: string;
} : {};
declare type ExtractRouteParams<Route extends string> = Route extends `${infer Path}?${infer Search}#${infer Hash}` ? ExtractPathParams<Path> & ExtractSearchParams<Search> & ExtractHashParam<Hash> : Route extends `${infer Path}?${infer Search}` ? ExtractPathParams<Path> & ExtractSearchParams<Search> : Route extends `${infer Path}#${infer Hash}` ? ExtractPathParams<Path> & ExtractSearchParams<Hash> : ExtractPathParams<Route>;
export declare type PrependBasePath<Routes extends Record<string, string>, BasePath extends string> = {
    [K in keyof Routes]: `${BasePath}/${Routes[K]}`;
};
export declare type GetNestedRoutes<Routes extends Record<string, string>> = {
    [K in keyof Routes as Routes[K] extends `${infer _}*` ? K : never]: Routes[K] extends `${infer Rest}*` ? Rest : never;
};
export declare type ExtractRoutesParams<Routes extends Record<string, string>> = {
    [K in keyof Routes]: ExtractRouteParams<Routes[K]>;
};
declare type EmptyRecord = Record<string | number | symbol, never>;
export declare type Simplify<T> = T extends EmptyRecord ? {} : {
    [K in keyof T]: T[K];
};
declare type NonOptionalProperties<T> = Exclude<{
    [K in keyof T]: T extends Record<K, T[K]> ? K : never;
}[keyof T], undefined>;
export declare type ParamsArg<Params> = Params extends EmptyRecord ? [] : NonOptionalProperties<Params> extends never ? [params?: {
    [K in keyof Params]: Params[K];
}] : [params: {
    [K in keyof Params]: Params[K];
}];
export {};
